/*
 *  bellevue.cpp
 *  Bellevue codec
 *
 *  Created by matti on 24.11.12
 *  Copyright 2012 Belle Nuit Montage. All rights reserved.
 *  http://www.belle-nuit.com/bellevue
 *
 */

#ifndef SFXTWL
#define SFXTWL

#include <iostream>
#include <fstream>
#include <string>
using namespace std;
#include <zlib.h>

#define limit255(x) ( x<0 ? 0 : (x > 255 ? 255 : x) )

//#define quantize(x,q,pivot) ( ( (x-pivot) / q ) * q + pivot )
#define quantize(x,q,pivot) ( x>pivot ? (((x-pivot+q/2) / q ) + pivot ) :  (((x-pivot-q/2) / q ) + pivot ) )
#define dequantize(x,q,pivot)  (((x-pivot) * q ) + pivot )

class sfxBellevue
	{
	public:
		char * filename;
		char * identdata;
		unsigned char identsize;
		unsigned char colourmaptype;
		unsigned char imagetype;
		short colourmapstart;
		short colourmaplength;
		short colourmapbits;
		short xstart;
		short ystart;
		short width;
		short height;
		unsigned char bits;
		unsigned char descriptor;
		unsigned long datalength;
		unsigned long compresseddatalength;
		unsigned long zrlelength;
		int validfile;
		int level;
		int quant;
		int interleaved;
		int rgb;
		int zlevel;
		int blockfilter;
		int version;
		int zrle;
		int sub422;
		int ampsep;
		int mainvalp;
		
		char * data;
		
		int init()
		{
			identdata = new char[32];
			identsize = 32;
			colourmaptype = 0;
			
			imagetype = 2;
			colourmapstart = 0;
			colourmaplength = 0;
			colourmapbits = 0;
			xstart = 0;
			ystart = 0;
			width = 0;
			height = 0;
			bits = 24;
			descriptor =  0;
			datalength = 0;
			validfile = 0;
			quant = 1;
			level = 1;
			interleaved = 1;
			zlevel = 1;
			blockfilter = 0;
			version = 1;
			data = NULL;
			zrle=0;
			sub422=0;
			ampsep = 0;
			mainvalp = 0;
			return 0;
			
		};
		sfxBellevue()
		{
			init();
		}
		sfxBellevue(const sfxTGA &original)
		{
			init();
			imagetype = original.imagetype;
			width = original.width;
			height =  original.height;
			bits =  original.bits;
			datalength =  original.datalength;
			data = new char [datalength];
			validfile =  original.validfile;
		};
		sfxBellevue(const int w, const int h)
		{
			init();
			width = w;
			height =  h;
			datalength = width*height*3;
			compresseddatalength = datalength;
		};
		
		
		~sfxBellevue() 
		{
			//if (width>0 && height>0 && data)
			//	delete[] data;
		
		};
		
		int readfile(char * fn)
		{
			short low, high, headerlength;
			
			filename = fn;
			
			 ifstream infile(fn, ifstream::binary);
			 
			 if (infile)
			 {
				 identsize = infile.get();
				 colourmaptype = infile.get();
				 imagetype = infile.get();
				 low = infile.get();
				 high = infile.get();
				 colourmapstart = low + 256 * high;
				 low = infile.get();
				 high = infile.get();
				 colourmaplength =  low + 256 * high;
				 colourmapbits =  infile.get();
				 low = infile.get();
				 high = infile.get();
				 xstart =  low + 256 * high;
				 low = infile.get();
				 high = infile.get();
				 ystart =  low + 256 * high;
				 low = infile.get();
				 high = infile.get();
				 width =  low + 256 * high;
				 low = infile.get();
				 high = infile.get();
				 height =  low + 256 * high;
				 bits =  infile.get();
				 descriptor = infile.get();
				 datalength = width*height*bits/8;
				 
				 if (imagetype == 2 && width>0 && height>0)
					 validfile = 1;
					 
				 if (identsize)
				 {
				 	identdata = new char[identsize];
				 	infile.read(identdata,identsize);
				 	
				 	for (int i=0;i<identsize-1;i++)
				 	{
				 		switch (identdata[i])
				 		{
				 			default: continue;
				 			case 'l' : level = identdata[i+1]; i++; break;
				 			case 'q' : quant = identdata[i+1]; i++; break;
				 			case 'i' : interleaved = identdata[i+1]; i++; break;
				 			case 'r' : rgb = identdata[i+1]; i++; break;
				 			case 'z' : zlevel = identdata[i+1]; i++; break;
				 			case 'Z' : zrle = identdata[i+1]; i++; break;
				 			case 's' : sub422 = identdata[i+1]; i++; break;
				 			case 'V' : version = identdata[i+1]; i++; break;
				 			case 'b' : blockfilter = identdata[i+1]; i++; break;
				 			case 'a' : ampsep = identdata[i+1]; i++; break;
				 		}
				 	
				 	
				 	}
				 	
				 	
				 }	
				 headerlength = infile.tellg();
				 
 				 data = new char [datalength]; 				 
 				 infile.seekg (0, ios::end);
    			 compresseddatalength = infile.tellg();
    			 compresseddatalength -= headerlength;
			 	 infile.seekg (headerlength);
			 	 
			 	 if (compresseddatalength>datalength)
			 	 	compresseddatalength=datalength;
 				 
				 infile.read(data,compresseddatalength);
				 infile.close(); 
			 }
			return 0;
			
		};
		
		int writefile(char * fn)
		{
			unsigned char low, high;
			
			filename = fn;
			
			ofstream outfile(filename, ofstream::binary);
			if (outfile)
			{
				unsigned char colourmapstartl = colourmapstart % 256;
				unsigned char colourmapstarth = colourmapstart >> 8 ;
				unsigned char colourmaplengthl = colourmaplength % 256;
				unsigned char colourmaplengthh = colourmaplength >> 8;
				unsigned char xstartl= xstart % 256;
				unsigned char xstarth = xstart >> 8;
				unsigned char ystartl= ystart % 256;
				unsigned char ystarth = ystart >> 8;
				unsigned char widthl= width % 256;
				unsigned char widthh = width >> 8;
				unsigned char heightl= height % 256;
				unsigned char heighth = height >> 8;
				
				identdata[0] = 'l';
				identdata[1] = level;
				identdata[2] = 'q';
				identdata[3] = quant;
				identdata[4] = 'i';
				identdata[5] = interleaved;
				identdata[6] = 'r';
				identdata[7] = rgb;
				identdata[8] = 's';
				identdata[9] = sub422;
				identdata[10] = 'b';
				identdata[11] = blockfilter;
				identdata[12] = 'z';
				identdata[13] = zlevel;
				identdata[14] = 'a';
				identdata[15] = ampsep;
				identdata[16] = 'Z';
				identdata[17] = zrle;
				identdata[18] = 'V';
				identdata[19] = version;
				
				for (int i = 20;i<identsize;i++)
				{
					identdata[i] = ' ';
				}
				
				
				outfile << identsize
				<< colourmaptype
				<< imagetype
				<<  colourmapstartl << colourmapstarth
				<<  colourmaplengthl << colourmaplengthh
				<<  char(colourmapbits)
				<<  xstartl << xstarth 
				<<  ystartl << ystarth
				<<  widthl << widthh
				<<  heightl << heighth
				<< char(bits)
				<< char(descriptor);
				outfile.write(identdata,identsize);
				outfile.write(data,compresseddatalength);
				outfile.close();
			}
			
			validfile = 1;
			return 0;
		};
		
		int decompress()
		{
			if (zlevel>0)
				inflate();
				
			if (zrle>0)
				zrledecompress();	
			
			if (ampsep)
				amplitudecomposition();	
				
			if (!interleaved)
				deplanar();
			return 0;
		};
		
		int compress()
		{
			if (!interleaved)
				planar();
			
			
			if (ampsep)
				amplitudeseparation();
				
				
			// count 80 or 88
			char * d = data;
			unsigned char cv= 128;
			unsigned char v;
			int count = 0;
			if (ampsep) 
				 cv = 136; 
			
			for (int i=0;i<datalength;i++)
			{
				v = (unsigned char)*d++;
				if (v==cv) count++;
				
			}
			mainvalp = 100 * count / datalength;
			
			

			if (zrle>0)
				zrlelength = zrlecompress();
			else
				zrlelength=datalength;
			
			
			if (zlevel>0)
				deflate();
				
			return 0;
		};
		
		int amplitudeseparation()
		{
		
			char * buffer = new char [datalength];
			char * d = data;
			char * blow = buffer;
			char * bhigh = buffer + datalength / 2;
			int value, value2, high, low;
			
			
			for (int i=0;i<datalength;i+=2)
			{
				value = (unsigned char)*d++;
				value2 = (unsigned char)*d++;
				
				//bias
				value += 8; if (value>255) value -= 256;
				value2 += 8; if (value2>255) value2 -= 256;
				
				
				high = (value & 0xF0) + ((value2 & 0xF0) >> 4);
				low = ((value & 0x0F) << 4) + (value2 & 0x0F);
				
				
				
				*bhigh++ = high;
				*blow++ = low;
			}
			
			memcpy (data, buffer,datalength);
			return 0;
		}

		int amplitudecomposition()
		{
		
			char * buffer = new char [datalength];
			char * dlow = data;
			char * dhigh = data + datalength / 2;
			char * b = buffer;
			int value, value2, high, low;
			
			for (int i=0;i<datalength;i+=2)
			{
				low = (unsigned char)*dlow++;
				high = (unsigned char)*dhigh++;
				
				
				
				value = (high & 0xf0) + ((low & 0xF0) >>4);
				value2 = ((high & 0x0f) << 4) + (low & 0x0f);
				
				//bias
				value -= 8; if (value<0) value += 256;
				value2 -= 8; if (value2<0) value2 += 256;
				
				*b++ = value;
				*b++ = value2;
			}
			
			memcpy (data, buffer,datalength);
			return 0;
		}

		int zrlecompress()
		{
			// conditional run length encoding: only 128 has run length
			char * buffer = new char [datalength];
			char * d = data;
			char * b = buffer;
			int value;
			unsigned char count=0;
		    int mainvalue = 128;
		    if (ampsep) mainvalue = 136;
			
			
			for (int i=0;i<datalength;i++)
			{
				if (b >= buffer + datalength - 2) 
				{
					// overrun, can happen with low compression, do not use zrle in this case
					zrle = 0;
					return datalength;
				}
				
				
				value = (unsigned char)*d++;
				
				if (value == mainvalue)
				{	
					if (count<255)
						count++;
					else
					{
						*b++ = mainvalue;
						*b++ = count;
						count = 1;
					}
					
				}
				else
				{
					
					 if (count>0)
					 {
					 	*b++ = mainvalue;
						*b++ = count;
					 }
					
					 
					*b++ = value;
					count = 0;
				}
				
			}
			
			
			if (count>0)
			{
				*b++ = mainvalue;
				*b++ = count;
				count = 0;
		    }
		    
			memcpy (data, buffer,datalength);
			return b-buffer;
		
		}
		
		int zrledecompress()
		{
		
			char * buffer = new char [datalength];
			char * d = data;
			char * b = buffer;
			unsigned char value;
			unsigned char count=0;
			int mainvalue = 128;
		    if (ampsep) mainvalue = 136;
						
			for (int i=0;i<datalength;i++)
			{
				if (b >= buffer + datalength) continue;
				if (d >= data + datalength) continue;
				
				value = (unsigned char)*d++;
				
				if (value == mainvalue)
				{
					count = (unsigned char)*d++;
					for(int j=0;j<count;j++) 
						*b++ = value;
				}
				else
				{
					*b++ = (unsigned char)value; 
				}
			}
			
			memcpy (data, buffer,datalength);
			return 0;
		
		}

		
		int deflate()
		{
			// using zlib compress2
			
			
			char * buffer = new char [datalength];
			
			char * d = data;
			char * b = buffer;
			uLongf * bufferlength  = new uLongf ;
			bufferlength[0] = datalength;
			
			int err = compress2((unsigned char*)b, bufferlength, 
                                (const unsigned char*)d, zrlelength,zlevel);
			compresseddatalength = bufferlength[0];
			
			
			
			memcpy (data, buffer,compresseddatalength);
			return 0;

		};
		
		int inflate()
		{
			// using zlib uncompress
			
			
			
			char * buffer = new char [datalength];
			char * d = data;
			char * b = buffer;
			uLongf * bufferlength = new uLongf ;
			bufferlength[0] = datalength;
			
			int err = uncompress((unsigned char*)b, bufferlength, 
                                (const unsigned char*)d, compresseddatalength);
			memcpy (data, buffer, datalength);
			return 0;
		};
		

		
		int planar()
		{
		
			char * buffer = new char [datalength];
			int channels = 3;
			int pixelcount = width*height;
			
			char * c = data;
			char * dr = buffer;
			char * dg = dr + pixelcount;
			char * db = dg + pixelcount;
			
			for (int x=0; x<pixelcount; x++)
			{
				*dr++ = *c++;
				*dg++ = *c++;
				*db++ = *c++;
			}
			
			memcpy (data, buffer, datalength);
			return 0;
		};
		
		int deplanar()
		{
		
			char * buffer = new char [datalength];
			int channels = 3;
			int pixelcount = width*height;
			
			char * c = buffer;
			char * dr = data;
			char * dg = dr + pixelcount;
			char * db = dg + pixelcount;
			
			for (int x=0; x<pixelcount; x++)
			{
				*c++ = *dr++;
				*c++ = *dg++;
				*c++ = *db++;
			}
			
			memcpy (data, buffer, datalength);
			return 0;
		};


		int rgb2yuv()
		{
		
			int channels = 3;
			int linebytes = width*channels;
			int r,g,b,yy1,yy2,cb1,cb2,cr1,cr2;
			
			for (int y=0;y<height;y++)
			{
				char * pixa = data + y * linebytes;
				
				for (int x=0;x<width;x+=2)
				{
					r = (unsigned char)pixa[0]; g = (unsigned char)pixa[1]; b = (unsigned char)pixa[2];
				    
				    
				    yy1 = ( 306 * r + 601 * g + 117 * b ) >> 10;
        			cb1 = (( -173 * r - 339 * g + 512 * b  ) >> 10) + 128;
       				cr1 = (( 512 * r - 429 * g - 83 * b  ) >> 10) + 128;
       				
       				r = (unsigned char)pixa[3]; g = (unsigned char)pixa[4]; b = (unsigned char)pixa[5];
				    
				    yy2 = ( 306 * r + 601 * g + 117 * b ) >> 10;
        			cb2 = (( -173 * r - 339 * g + 512 * b  ) >> 10) + 128;
       				cr2 = (( 512 * r - 429 * g - 83 * b  ) >> 10) + 128;
				    
				   
       				if (sub422)
       				{
       					cb1 = (cb1 + cb2 + 1) >>1; cb2 = cb1;
       					cr1 = (cr1 + cr2 + 1) >>1; cr2 = cr1;
       				}
       				
       				
        			pixa[0] = limit255(cr1); pixa[1] = limit255(yy1); pixa[2] = limit255(cb1);
        			pixa[3] = limit255(cr2); pixa[4] = limit255(yy2); pixa[5] = limit255(cb2);
        			
        			pixa += 2*channels;
				}
			}
			
			return 0;
		}
		
		int yuv2rgb()
		{
		
			int channels = 3;
			int linebytes = width*channels;
			int r,g,b,yy,cb,cr;
			
			for (int y=0;y<height;y++)
			{
				char * pixa = data + y * linebytes;
				
				for (int x=0;x<width;x++)
				{
					cr = (unsigned char)pixa[0]; yy = (unsigned char)pixa[1]; cb = (unsigned char)pixa[2];
				    
				    
				    r = yy + ((1437 * (cr - 128)) >> 10);
				    g = yy + ((-352 * (cb - 128) - 731 * (cr - 128)) >> 10);
				    b = yy + ((1815 * (cb - 128)) >> 10);
				    
				    /*
				    r = yy + 1.403 (cr - 128)
				    g = yy - 0.344 (cb - 128) - 0.714 (cr - 128)
				    r = yy + 1.773 (cr - 128)
       				*/
       				
        			pixa[0] = limit255(r); pixa[1] = limit255(g); pixa[2] = limit255(b);
        			
        			pixa += channels;
				}
			}
						
			return 0;
		}
		
		
		int transform()
		{
			/* 
				a b    m x
				c d => y t
			*/
							
			char * buffer = new char [datalength];
			int channels = 3;
			int linebytes = width*channels;
			int a,b,c,d;
			int m,x,y,t;
			int mt,ml,mr,mb,ar,br,cr,dr,ab,bb,cb,db;
			
			if (!rgb)
				rgb2yuv();
			
			
			for (int i=0;i<height;i+=2)
			{
				
				char * pixa = data + i * linebytes;
				char * pixb = data + i * linebytes + channels;
				char * pixc = data + (i + 1) * linebytes;
				char * pixd = data + (i + 1) * linebytes + channels;
				
				char * pixm = buffer + (i / 2) * linebytes;
				char * pixx = buffer + (i / 2) * linebytes + (width / 2 ) * channels;
				char * pixy = buffer + (i / 2) * linebytes + (height / 2 ) * linebytes;
				char * pixt = buffer + (i / 2) * linebytes + (height / 2 ) * linebytes + (width / 2) * channels;
				
				for (int j=0;j<width;j+=2)
				{
					// blockfilter
					
					
					
					
					for (int ch=0;ch<channels;ch++)
					{
					
						a = (unsigned char)pixa[ch]; 
						b = (unsigned char)pixb[ch]; 
						c = (unsigned char)pixc[ch]; 
						d = (unsigned char)pixd[ch]; 
						
						m = a + b + c + d + 2 >> 2;
						x = a - b + c - d + 514 >> 2;
						y = a + b - c - d + 514 >> 2;					
						t = a - b - c + d + 514 >> 2;
						
						if (blockfilter && i>0 && i<height-2 && j>0 && j<width-2)
						{
							
							char * pixmt = pixm-linebytes;
							char * pixml = pixm-channels;
		
							char * pixab = pixa + 2 * linebytes; 
							char * pixbb = pixb + 2 * linebytes; 
							char * pixcb = pixc + 2 * linebytes; 
							char * pixdb = pixd + 2 * linebytes; 
							
							char * pixar = pixa + 2*channels;
							char * pixbr = pixb + 2*channels;
							char * pixcr = pixc + 2*channels;
							char * pixdr = pixd + 2*channels; 
							
							ar = (unsigned char)pixar[ch]; 
							br = (unsigned char)pixbr[ch]; 
							cr = (unsigned char)pixcr[ch]; 
							dr = (unsigned char)pixdr[ch]; 
							ab = (unsigned char)pixab[ch]; 
							bb = (unsigned char)pixbb[ch]; 
							cb = (unsigned char)pixcb[ch]; 
							db = (unsigned char)pixdb[ch]; 
							
							mt = (unsigned char)pixmt[ch]; 
							ml = (unsigned char)pixml[ch]; 
							mr = ar + br + cr + dr + 2 >> 2;
							mb = ab + bb + cb + db + 2 >> 2;
							
							x -= (ml - mr) >> 3;
							y -= (mt - mb) >> 3;
							
						}
						
						if (quant>1) 
						{
							x = quantize(x,quant,128);
							y = quantize(y,quant,128);
							t = quantize(t,quant,128);
						}

						pixm[ch] = limit255(m); 
						pixx[ch] = limit255(x);  
						pixy[ch] = limit255(y); 
						pixt[ch] = limit255(t); 
				  		
				  	}
				  	
				  	pixa += channels*2;
				  	pixb += channels*2;
				  	pixc += channels*2;
				  	pixd += channels*2;
				  	
				  	pixm += channels;
				  	pixx += channels;
				  	pixy += channels;
				  	pixt += channels;
				  	
				}
			
			}
			
			if (level > 1)
			{
				sfxBellevue tinyfile;
       			tinyfile.width = width >> 1;
        		tinyfile.height = height >> 1;
        		tinyfile.datalength = datalength >> 2;
        		tinyfile.data = new char [tinyfile.datalength];
        		tinyfile.quant = quant >> 1;
        		tinyfile.level = level - 1;
        		tinyfile.blockfilter = blockfilter;
        		tinyfile.rgb = 1; // make conversion only on top level
        		
        		// copy to tiny
        		
				for (int y=0;y<tinyfile.height;y++)
				{
					char * pixa = buffer + y * linebytes;
					char * pixm = tinyfile.data + y * (linebytes >> 1);
				
					for (int x=0;x< tinyfile.width*channels;x++)
					{
						*pixm = *pixa;
						pixm++;
						pixa++;
        			}
        		}
        		
        		tinyfile.transform();
        		
        		// copy back
        		
        		for (int y=0;y<tinyfile.height;y++)
				{
					char * pixa = buffer + y * linebytes;
					char * pixm = tinyfile.data + y * (linebytes >> 1);
				
					for (int x=0;x< tinyfile.width*channels;x++)
					{
						*pixa = *pixm;
						pixm++;
						pixa++;
        			}
        		}
        	}
			
			
			
			// copy back
			
			memcpy (data, buffer, datalength);
			return 0;
		}
		
		
		int detransform()
		{
						
			char * buffer = new char [datalength];
			int channels = 3;
			int linebytes = width*channels;
			int a,b,c,d;
			int m,x,y,t,mt,ml,mb,mr;
			 
			
			if (level > 1)
			{
				sfxBellevue tinyfile;
       			tinyfile.width = width >> 1;
        		tinyfile.height = height >> 1;
        		tinyfile.datalength = datalength >> 2;
        		tinyfile.data = new char [tinyfile.datalength];
        		tinyfile.quant = quant >> 1;
        		tinyfile.level = level - 1;
        		tinyfile.blockfilter = blockfilter; 
        		tinyfile.rgb = 1; // make conversion only on top level
        		
        		// copy to tiny
        		
				for (int y=0;y<tinyfile.height;y++)
				{
					char * pixa = data + y * linebytes;
					char * pixm = tinyfile.data + y * (linebytes >> 1);
				
					for (int x=0;x< tinyfile.width*channels;x++)
					{
						*pixm = *pixa;
						pixm++;
						pixa++;
        			}
        		}
        		
        		tinyfile.detransform();
        		
        		// copy back
        		
        		for (int y=0;y<tinyfile.height;y++)
				{
					char * pixa = data + y * linebytes;
					char * pixm = tinyfile.data + y * (linebytes >> 1);
				
					for (int x=0;x< tinyfile.width*channels;x++)
					{
						*pixa = *pixm;
						pixm++;
						pixa++;
        			}
        		}
        		
        		
        	}
				
			for (int i=0;i<height;i+=2)
			{
				char * pixm = data + (i >> 1) * linebytes;
				char * pixx = pixm + (width >> 1) * channels;
				char * pixy = pixm + (height >> 1) * linebytes;
				char * pixt = pixy + (width >> 1) * channels;
				
				char * pixa = buffer + i * linebytes;
				char * pixb = pixa + channels; 
				char * pixc = pixa + linebytes; 
				char * pixd = pixa + linebytes + channels; 
								
				for (int j=0;j<width;j+=2)
				{
					for (int ch=0;ch<channels;ch++)
					{

						m = (unsigned char)pixm[ch]; 
						x = (unsigned char)pixx[ch]; 
						y = (unsigned char)pixy[ch]; 
						t = (unsigned char)pixt[ch];
						
						if (quant>1) 
						{
							x = dequantize(x,quant,128);
							y = dequantize(y,quant,128);
							t = dequantize(t,quant,128);
						}
						
						
						if (blockfilter && i>0 && i<height-2 && j>0 && j<width-2)
						{
							char * pixmt = pixm-linebytes;
							char * pixml = pixm-channels;
							char * pixmb = pixm+linebytes;
							char * pixmr = pixm+channels;
							
														
							mt = (unsigned char)pixmt[ch]; 
							ml = (unsigned char)pixml[ch]; 
							mb = (unsigned char)pixmb[ch]; 
							mr = (unsigned char)pixmr[ch]; 

							
							x += (ml - mr + 2) >> 3;
							y += (mt - mb + 2) >> 3;
							
							
							
							
							
													
						}
						
						a = m + x + y + t - 384;      
						b = m - x + y - t + 128;
						c = m + x - y - t + 128; 
						d = m - x - y + t + 128; 
						

						pixa[ch] = limit255(a); 
						pixb[ch] = limit255(b); 
						pixc[ch] = limit255(c); 
						pixd[ch] = limit255(d); 
						
						
				  	
				  	}
				  	
				  	pixa += channels*2;
				  	pixb += channels*2;
				  	pixc += channels*2;
				  	pixd += channels*2;
				  	
				  	pixm += channels;
				  	pixx += channels;
				  	pixy += channels;
				  	pixt += channels;
				}
			
			}
			
			
			
			
			
			
			memcpy (data, buffer, datalength);

			if (!rgb)
				yuv2rgb();
			
			return 0;
		}
		
		
		int dump()
		{
			
			float bitsperpixel = 8.0 * compresseddatalength / ( width * height );
			char * buffer = new char[10];
			sprintf(buffer,"%1.1f",bitsperpixel);
			
			float percent = 100.0 * compresseddatalength / datalength;
			char * buffer2 = new char[10];
			sprintf(buffer2,"%1.1f",percent);
						
			cout << "BLV " << filename; 
			if (validfile)
				cout << " valid ";
			else 
				cout << " invalid ";
			cout << width << "x" << height << " ";
			cout << compresseddatalength << "(" << buffer2 << " % " << zrlelength << " " << mainvalp << "%)"
			  << " bytes " << buffer << " bpp" 
			  << " (-l " << level << " -q " << quant << " -i "  << interleaved 
			  << " -r " << rgb   << " -s " << sub422 
			  << " -z "  << zlevel << " -Z "  << zrle << " -a "  << ampsep
			  << " -b " << blockfilter
			  << " -V "  << version
			  << ")" << endl;
			 
			return 0;
		};
		
	};

#endif


