//
//  main.cpp
//  bellevue
//
//  Created by matti on 24.11.12.
//  Copyright (c) 2012 matti. All rights reserved.
// 

//  needs zlib.h and libz.a in the folder
// 	c++ libz.a main.cpp -o bellevue -O2
//

#include <iostream>
#include <string>
#include <sys/time.h>
#include "tga.cpp" 
#include "bellevue.cpp" 
#include "zlib.h"


void usage_exit(void)
{
	
    std::cout << "Bellevue by Matthias Buercher v1.0\n"
"(c) 2012 matti@belle-nuit.com \n"
"Demonstrates the Bellevue compression\n"
"\n"
"usage: bellevue [-options] infile.tga outfile.bvl\n"
"usage: bellevue -d [-options] infile.bvl outfile.tga\n"
"  -v verbose operation\n"
"  -d decompress\n"
"  -l level (0-4, default 1)\n"
"  -q quantize (1-32, default 1)\n"
"  -z zlib compresssion lebel (0-9, default 1)\n"
"  -Z use zrle\n"
"  -a use amplitude separation\n"
"  -r use rgb, do not convert to/from yuv\n"
"  -s subsample 422 (only with yuv)\n"
"  -i use interleaved pixels, do not planar\n"
"  -b use blockfilter on decompression\n"
"  infile.tga TGA file must be RGB 24bit uncompressed and dimensions divisible by pow(2,level)\n"
"  outfile.bvl Bellevue file\n"
"  infile.bvl Bellevue file\n"
"  outfile.tga TGA\n"
"  most compression -l 4 -q 32 -z 9\n"
;

	exit(2);
	
}

int main (int argc, char * const argv[]) {
    ++argv;
	--argc;
	
	
	int verbose=0;
	int decompress=0;
	int level=1;
	int quant=1;
	int zlevel=1;
    int rgb=0;
    int sub422=0;
    int interleaved=0;
    int inputwidth=0;
	int inputheight=0;
	int blockfilter=0;
	int zrle=0;
	int ampsep = 0;
	clock_t clockstart, clockend, clockfirststart, clocklastend;
	
	clockfirststart = clock();

    	
	while (*argv && **argv=='-')
    {
		char c = *(*argv + 1);
		switch (c)
        {
			default: usage_exit(); break;
			case 'v': verbose = 1; break;
			case 'd': decompress = 1; break;
			case 'l': ++argv; --argc; level = atoi(*argv); break;
			case 'q': ++argv; --argc; quant = atof(*argv); break;
			case 'z': ++argv; --argc; zlevel = atof(*argv); break;
			case 'r': rgb = 1; break;
			case 'i': interleaved = 1; break;
			case 's': sub422 = 1; break;
			case 'a': ampsep = 1; break;
			case 'b': blockfilter = 1; break;
			case 'Z': zrle = 1; break;
        }
		++argv;
		--argc;
    }
    
    
    
    if (argc==0 || argc==1) usage_exit();
    
    char * filename = argv[0];
    char * outfilename = argv[1];
    
    if (decompress)
    {
        
        sfxBellevue tinyfile;
        
        if (verbose)
        {
            cout << "--- bellevue decompress---" << endl;
            cout << "reading... ";
            clockstart = clock();
        }
        
        tinyfile.readfile(filename);
        
        inputwidth = tinyfile.width;
        inputheight = tinyfile.height;
        
        sfxTGA tgafile;
        
        tgafile.width = tinyfile.width;
        tgafile.height = tinyfile.height;
        tgafile.datalength = tinyfile.height*tinyfile.width*3;
		tgafile.bits = 24;
		tgafile.imagetype = 2;
        
        quant = tinyfile.quant;
        level = tinyfile.level;
        zlevel = tinyfile.zlevel;
        rgb = tinyfile.rgb;
        // tinyfile.blockfilter = blockfilter;
        
        if (verbose)
		{
			clockend = clock();
			cout << (clockend-clockstart) * 1000 / CLOCKS_PER_SEC << endl;
			tinyfile.dump();
        }
        
        if (zlevel>0)
        {
			if (verbose)
				cout << "decompressing...";
			
		}
		
		clockstart = clock();
		tinyfile.decompress();
			
			clockend = clock();
			if (verbose)
				cout << (clockend-clockstart) * 1000 / CLOCKS_PER_SEC << " ";
			
        
        
        if (level>0)
        {
			if (verbose)
			{
				cout << "detransforming...";
			}
			clockstart = clock();
	
			tinyfile.detransform();  
			
			clockend = clock();
			if (verbose)
				cout << (clockend-clockstart) * 1000 / CLOCKS_PER_SEC << " ";
        }
        
        
        
              
        
       	if (verbose)
        {
        	cout << "writing..."<<endl;
        }
        clockstart = clock();
        
        tgafile.data = tinyfile.data;
        tgafile.writefile(outfilename);
 
        clockend = clock();
 		
 		if (verbose)
        {
        	cout << (clockend-clockstart) * 1000 / CLOCKS_PER_SEC << endl;
        	tgafile.dump();	
        	
        	clocklastend = clock();
        	cout << "complete ";
        	cout << (clocklastend-clockfirststart) * 1000 / CLOCKS_PER_SEC << " msec"<< endl;
            cout << "------"<< endl;
        }
        
    }
    else
    {
        
        if (verbose)
        {
        	cout << "--- bellevue---" << endl;
        	cout << "reading...";
        }
        clockstart = clock();
                
        sfxTGA tgafile;
        tgafile.readfile(filename);

        inputwidth = tgafile.width;
        inputheight = tgafile.height;
        	
        int divider = 2; 
		for (int j=1;j<level;j++) divider *= 2;
	
		if (inputwidth % divider != 0)
		{
			cout << "ERROR: Width must be divisible by " << divider << endl;
			return -1;
		}
		
		if (inputheight % divider != 0)
		{
			cout << "ERROR: Width must be divisible by " << divider << endl;
			return -1;
		}
			
     	clockend = clock();
     	
     	if (verbose)
     	{
     		cout << (clockend-clockstart) * 1000 / CLOCKS_PER_SEC << endl;
     		tgafile.dump();
     	}
     	
        sfxBellevue tinyfile(tgafile.width,tgafile.height);
                
        tinyfile.data = tgafile.data;
        tinyfile.quant = quant;
        tinyfile.level = level;
        tinyfile.interleaved = interleaved;
        tinyfile.rgb = rgb;
        tinyfile.zlevel = zlevel; 
        tinyfile.zrle = zrle;
        tinyfile.sub422 = sub422;
        tinyfile.ampsep = ampsep;
        tinyfile.blockfilter = blockfilter;
                
        if (level > 0)
        {
        	if (verbose)
        		cout << "transforming...";
       		clockstart = clock();
       	
       	
       		tinyfile.transform(); 
       	
       		clockend = clock();
			if (verbose)
     			cout << (clockend-clockstart) * 1000 / CLOCKS_PER_SEC << " ";
       	}
       	 
       	if (verbose)
       		cout << "compressing...";
       	clockstart = clock();	
       		
		tinyfile.compress();
       	
		clockend = clock();
		if (verbose)
     		cout << (clockend-clockstart) * 1000 / CLOCKS_PER_SEC << " ";
       	
       	if (verbose)
        	cout << "writing...";
       	clockstart = clock();	
       		
		tinyfile.writefile(outfilename);
       	
		clockend = clock();
		if (verbose)
     		cout << (clockend-clockstart) * 1000 / CLOCKS_PER_SEC << " ";


       
        if (verbose)
        {
        	cout << endl;
        	tinyfile.dump();
        	cout << "complete ";
        	clocklastend = clock();
        	cout << (clocklastend-clockfirststart) * 1000 / CLOCKS_PER_SEC << " msec"<< endl;
        	
        	
            cout << "------" << endl;
        }
   
		
        
    }
    
    
    return 0;
}




