//
//  mainpsnr.cpp
//  psnr
//
//  Created by matti on 15.12.12.
//  Copyright (c) 2012 matti. All rights reserved.
// 

// 	c++ mainpsnr.cpp -o psnr -O2
//

#include <iostream>
#include <string>
#include <sys/time.h>
#include "tga.cpp" 
#include "psnr.cpp" 


void usage_exit(void)
{
	
    std::cout << "psnr by matthias buercher v1.0\n"
"(c) 2012 matti@belle-nuit.com \n"
"calculates PSNR between two TGA files\n"
"\n"
"usage: psnr [-v] file0.tga file1.tga\n"
"  -v verbose operation\n"
"  -r rgb operation\n"
"  TGA file must be RGB 24bit uncompressed or RLE\n"
"  both files must have the same dimensions\n"
;

	exit(2);
	
}

int main (int argc, char * const argv[]) {
    ++argv;
	--argc;
	
	
	int verbose=0;
	int inputwidth = 0;
	int inputheight = 0;
	clock_t clockstart, clockend, clockfirststart, clocklastend;
	
	clockfirststart = clock();

    	
	while (*argv && **argv=='-')
    {
		char c = *(*argv + 1);
		switch (c)
        {
			default: usage_exit(); break;
			case 'v': verbose = 1; break;
        }
		++argv;
		--argc;
    }
    
    if (argc==0 || argc==1) usage_exit();
    
    char * filename0 = argv[0];
    char * filename1 = argv[1];
    
    
    if (verbose)
    {
        cout << "--- psnr---" << endl;
        cout << "reading...";
    }
    
    
    
	clockstart = clock();
			
	sfxTGA tgafile0;
	tgafile0.readfile(filename0);

	inputwidth = tgafile0.width;
	inputheight = tgafile0.height;
	
	
		
	sfxTGA tgafile1;
	tgafile1.readfile(filename1);
	
	

	if (inputwidth != tgafile1.width)
	{
		cout << "ERROR: Width does not match " << inputwidth << " != " << tgafile1.width << endl;
		return -1;
	}
	if (inputheight != tgafile1.height)
	{
		cout << "ERROR: Height does not match " << inputheight << " != " << tgafile1.height << endl;
		return -1;
	}
	clockend = clock();
	
	if (verbose)
	{
		cout << (clockend-clockstart) * 1000 / CLOCKS_PER_SEC << endl;
		tgafile0.dump();
		tgafile1.dump();
	}
	
	
	
	sfxPSNR processor(tgafile0.width,tgafile1.height);
	processor.data0 = tgafile0.data;
	processor.data1 = tgafile1.data;
	
	
	
	if (verbose)
		cout << "analyzing...";
	clockstart = clock();
	
	
	float result = processor.getpsnr();
	
	sfxTGA tgafile;
	tgafile.width=inputwidth;
	tgafile.height=inputheight;
	tgafile.datalength=inputheight*inputwidth*3;
	tgafile.data = processor.diffdata;
	
	
	char * fn = new char[255];
	sprintf(fn, "%s%s",filename1,"-diff.tga");
	tgafile.writefile(fn);
	// tgafile.dump();
	
	
	clockend = clock();
	if (verbose)
	{
		cout << (clockend-clockstart) * 1000 / CLOCKS_PER_SEC << " " << endl;
		
	}
	
	char * restext = new char[32];
	sprintf(restext,"%0.2f dB",result);
	cout << "PSNR = " << restext ;
	
	 result = processor.getssim();
	sprintf(restext,"%0.3f",result);
	cout << " " << restext << endl;
	
	;
	
}




