/*
 *  psnr.cpp
 *  PSNR calculation
 *
 *  Created by matti on 15.12.12
 *  Copyright 2012 Belle Nuit Montage. All rights reserved.
 *  http://www.belle-nuit.com/tiny-wavelet
 *
 */

#ifndef SFXPSNR
#define SFXPSNR

#include <iostream>
#include <fstream>
#include <string>
#include <math.h>
using namespace std;

#define limit255(x) ( x<0 ? 0 : (x > 255 ? 255 : x) )

// #define limit255(x) (x)

#define quantize(x,q,pivot) ( ( (x-pivot) / q ) * q + pivot )


#define scale(x) (x >> 2)
#define unscale(x) (x << 2)

class sfxPSNR
	{
	public:
		short width;
		short height;
		char * data0;
		char * data1;
		char * diffdata;
		
		int init()
		{
			width = 0;
			height = 0;
			data0 = NULL;
			data1 = NULL;
			return 0;
		};
		sfxPSNR()
		{
			init();
		}
		sfxPSNR(const int w, const int h)
		{
			init();
			width = w;
			height =  h;
			diffdata = new char[width*height*3];
		};
		
		
		~sfxPSNR() 
		{
		};
		


		float getpsnr()
		{
			
			
			if (data0 == NULL || data1 == NULL) return -1.0;
			
			int channels = 3;
			int linebytes = width*channels;
			float r0,g0,b0,yy0;
			float r1,g1,b1,yy1;
			float rd,gd,bd;
			float mse=1.0; // should not be zero;
			float peak;
			float psnr, psnrdb;
			
			
			
			for (int y=0;y<height;y++)
			{
				char * pix0 = data0 + y * linebytes;
				char * pix1 = data1 + y * linebytes;
				char * pix = diffdata + y * linebytes;
				
				
				
				for (int x=0;x<width;x++)
				{
					r0 = (unsigned char)pix0[0]; g0 = (unsigned char)pix0[1]; b0 = (unsigned char)pix0[2];
					r1 = (unsigned char)pix1[0]; g1 = (unsigned char)pix1[1]; b1 = (unsigned char)pix1[2];
				    
				    
				    yy0 = ( 306 * r0 + 601 * g0 + 117 * b0 ) / 1024;
				    yy1 = ( 306 * r1 + 601 * g1 + 117 * b1 ) / 1024;

					mse += (yy1-yy0)*(yy1-yy0);
					
					rd = r1 - r0 + 128;
					gd = g1 - g0 + 128;
					bd = b1 - b0 + 128;
					pix[0] = limit255(rd);
					pix[1] = limit255(gd);
					pix[2] = limit255(bd);
        			
        			pix0 += channels;
        			pix1 += channels;
        			pix += channels;
        			
        			
				}
			}
			
			
			mse /= width * height;
			
			// cout <<  " mse " << mse << " ";
			
			peak = 255*255;
			psnr = peak / mse;
			
			psnr = 10 * log10(psnr);
			
			// cout << "(" << psnr << ")";
			return psnr;
		}
		
		float getssim()
		{
			
			
			if (data0 == NULL || data1 == NULL) return -1.0;
			
			int channels = 3;
			int linebytes = width*channels;
			float r0,g0,b0,yy0;
			float r1,g1,b1,yy1;
			float sumssim=0;
			int count = 0;
			
			for (int yb=0;yb<height;yb+=8)
			{
			
			for (int xb=0;xb<width;xb+=8)
			{
			
			
			float sumx=0.0;
			float sumy=0.0;
			float sumx2=0.0;
			float sumy2=0.0;
			float sumxy= 0.0;
			
			for (int y=0;y<8;y++)
			{
				char * pix0 = data0 + (yb + y) * linebytes + xb*channels;
				char * pix1 = data1 + (yb + y) * linebytes + xb*channels;
				
				for (int x=0;x<8;x++)
				{
					r0 = (unsigned char)pix0[0]; g0 = (unsigned char)pix0[1]; b0 = (unsigned char)pix0[2];
					r1 = (unsigned char)pix1[0]; g1 = (unsigned char)pix1[1]; b1 = (unsigned char)pix1[2];
				    
				    
				    yy0 = ( 306 * r0 + 601 * g0 + 117 * b0 ) / 1024;
				    yy1 = ( 306 * r1 + 601 * g1 + 117 * b1 ) / 1024;

					sumx += yy0;
					sumy += yy1;
					sumx2 += yy0*yy0;
					sumy2 += yy1*yy1;
					sumxy += yy0*yy1;
        			
        			pix0 += channels;
        			pix1 += channels;
        			
        			
        			
				}
			}
			
			float meanx = sumx / 64;
			float meany = sumy / 64;
			float varx = sumx2 / 64 - sumx*sumx/64/64;
			float vary = sumy2 / 64 - sumy*sumy/64/64;
			float covar = sumxy / 64 - sumx*sumy/64/64 ;
			float c1 = 0.01*255.0;
			float c2 = 0.03*255.0;
			
			
			
			float ssim= ( ( 2 * meanx * meany + c1 ) * ( 2 * covar + c2 ) ) / 
			 (( meanx*meanx + meany*meany + c1 ) * ( varx + vary + c2 ));
			 

			
			
			// cout << ssim << " ";
			sumssim += ssim;
			count++;
			
			//cout << "meanx " << meanx << " meany " << meany << " varx " << varx << " vary " << vary << " covar " << covar;
			// return ssim;
			
			}
			
			
			}
			
			// cout << "ssim";
			
			
			return sumssim/count;
		}
		
		

		
	};

#endif


